# Bipedal Walker Evo

This project tries to solve OpenAI's bipedal walker using three different ways: Q-Learning, Mutation of Actions and Evolution Strategies.

# Q-Learning
Coming soon

# Action Mutation
Will get 0 reward, which is basically learning to prevent falling on it's head.

## How it works
1. Generate a population with a starting number randomized actions (we don't need enough actions to solve the problem right now)
2. Let the population play the game reward every walker of the generation accordingly
3. The best walker survives without mutating
3. The better the reward the higher the chance to pass actions to next generation. Each child has a single parent, no crossover.
4. Mutate all children and increment their number of actions

## Hyperparameters
| Parameter         | Description                                                 | Interval  |
|-------------------|-------------------------------------------------------------|-----------|
| `POP_SIZE`        | Size of population.                                         | [0;∞[     |
| `MUTATION_FACTOR` | Percentage of weights that will be mutated for each mutant. | [0;1]     |
| `ACTIONS_START`   | Number of actions in the first generation.                  | [0;1600]  |
| `INCREASE BY`     | Incrementation of steps for each episode.                   | [0;∞[     |
| `MAX_STEPS`       | Number of steps that are played in one episode.             | [0; 1600] |

# Evolution Strategies
After 1000 episodes, which is about 1h of learning, it will reach ~250 reward.\
Best score until now: 292/300 in 7000 episodes \
![Reward](./EvolutionStrategies/Experiments/12_1_50_0.1_decaying_300/12_2_50_0.1_decaying_300.png)

## How it works
1. Generate a randomly weighted neural net
2. Create a population of neural nets with mutated weights
3. Let every net finish an episode and reward it accordingly
4. The better the reward, the higher the chance to pass weights to next gen

## Hyperparameters
| Parameter         | Description                                                 | Interval  |
|-------------------|-------------------------------------------------------------|-----------|
| `HIDDEN_LAYER`    | Size of hidden layer.                                       | [1;∞[     |
| `BIAS`            | Add a bias neuron to the input layer.                       | {0,1}     |
| `POP_SIZE`        | Size of population.                                         | [0;∞[     |
| `MUTATION_FACTOR` | Percentage of weights that will be mutated for each mutant. | [0;1]     |
| `LEARNING_RATE`   | This is the rate of learning.                               | [0;1]     |
| `GENS`            | Number of generations.                                      | [0;∞[     |
| `MAX_STEPS`       | Number of steps that are played in one episode.             | [0; 1600] |


# Installation
We use Windows, Anaconda and Python 3.7 \
`conda create -n evo_neuro python=3.7` \
`conda activate evo_neuro`\
`conda install swig`\
`pip install -r requirements.txt`



# Important Sources
Environment: https://github.com/openai/gym/wiki/BipedalWalker-v2 \
Table of all Environments: https://github.com/openai/gym/wiki/Table-of-environments
OpenAI Website: https://gym.openai.com/envs/BipedalWalker-v2/ \
More on evolution strategies: https://openai.com/blog/evolution-strategies/
